﻿"""
Sample TBC mcro to name line objects based on layer name
"""
#region
import clr
# Reference the WPF assemblies
clr.AddReference('IronPython.Wpf')
import wpf
from System.IO import StreamReader
from System.Windows.Controls import StackPanel
from System.Windows import  Visibility
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap
clr.AddReference("Trimble.DiskIO")
from Trimble.DiskIO import OptionsManager
clr.AddReference("Trimble.Vce.Core")
from Trimble.Vce.Core.Components import WorldView, Project, Layer
from Trimble.Vce.Core.Components.TextStyle import TextHeightMode, TextJustification
from Trimble.Vce.Core import TransactMethodCall
clr.AddReference("Trimble.Vce.Geometry")
from Trimble.Vce.Geometry import Point3D, Line2D, Side, Vector3D
from Trimble.Vce.Geometry.PolySeg import PolySeg
from Trimble.Vce.Geometry.PolySeg.Segment import Segment, Line
clr.AddReference("Trimble.Vce.Interfaces")
from Trimble.Vce.Interfaces.SnapIn import IPolyseg, IViewMember, ISettableName, IName, IViewSite
from Trimble.Vce.Interfaces.Client import CommandGranularity
clr.AddReference("Trimble.Vce.UI.Controls")
from Trimble.Vce.UI.Controls import GlobalSelection
clr.AddReference("Trimble.Vce.ForeignCad")
from Trimble.Vce.ForeignCad import TextEntity, Leader, MText
clr.AddReference("Trimble.Vce.UI.BaseCommands")
from Trimble.Vce.UI.BaseCommands import SelectionContextMenuHandler
clr.AddReference("Trimble.Vce.UI.UIManager")
from Trimble.Vce.UI.UIManager import UIEvents, UIEventArgs
clr.AddReference("Trimble.Vce.UI.Wrapper")
from Trimble.Vce.UI.Wrapper import CADWrapperHelper
#endregion
# define the command key and name
def Setup(cmdData, macroFileFolder):
    cmdData.Key = "AssignNameFromLayer"
    cmdData.CommandName = "AssignNameFromLayer"
    cmdData.Caption = "_Assign Name From Layer"
    cmdData.UIForm = "AssignNameFromLayer"
    #cmdData.HelpFile="Macros.chm"
    #cmdData.HelpTopic="????"

    cmdData.DefaultTabKey = "Macros"
    cmdData.DefaultTabGroupKey = "CAD"
    cmdData.ShortCaption = "Name from Layer"
    cmdData.ToolTipTittle = "Assign Names from Layer"
    cmdData.ToolTipTextFormatted = "Tool to name line object based on name of their layer."
    try:
        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass 

class AssignNameFromLayer(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\AssignNameFromLayer.xaml") as s:
            wpf.LoadComponent(self, s)
        self.currentProject = currentProject

    def OnLoad(self, cmd, buttons, event):
        self.Caption = cmd.Command.Caption
        self.objs.IsEntityValidCallback=self.IsValid
        optionMenu = SelectionContextMenuHandler();
        # remove options that don't apply here
        optionMenu.ExcludedCommands = "SelectObservations | SelectPoints | SelectDuplicatePoints";
        self.objs.ButtonContextMenu = optionMenu;
        self.lType = clr.GetClrType(IPolyseg)
        self.ignoreNamedCtl.IsChecked = OptionsManager.GetBool("AssignNameFromLayer:ignoreNamedCtl", True)
        self.nameMaskCtl.Text = OptionsManager.GetString("AssignNameFromLayer:nameMaskCtl", "{L}")
        self.labelLinesCtl.IsChecked = OptionsManager.GetBool("AssignNameFromLayer:labelLinesCtl", False)
        self.labelLinesCtl.Checked += self.labelLinesCtlChanged
        self.labelLinesCtl.Unchecked += self.labelLinesCtlChanged
        self.labelLinesCtlChanged(None, None)

    def labelLinesCtlChanged(self, sender, args):
        if self.labelLinesCtl.IsChecked:
            self.labelingControls.Visibility = Visibility.Visible
        else:
            self.labelingControls.Visibility = Visibility.Collapsed

    def CancelClicked(self, cmd, args):
        cmd.CloseUICommand()

    def IsValid(self, serial):
        o=self.currentProject.Concordance.Lookup(serial)
        if isinstance(o, self.lType):
            return True
        return False

    def OkClicked(self, cmd, e):
        # tell undo manager we are starting to do something
        self.currentProject.TransactionManager.AddBeginMark(CommandGranularity.Command, self.Caption)
        wv = self.currentProject[Project.FixedSerial.WorldView]
        UIEvents.RaiseBeforeDataProcessing(self, UIEventArgs())
        # pause the graphics so every change doesn't draw
        wv.PauseGraphicsCache(True)
        try:
            self.worldPlotscale = wv.PlotScale
            layers = self.currentProject[Project.FixedSerial.LayerContainer]
            # the "with" statement will unroll any changes if something go wrong
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
                for o in self.objs.SelectedMembers(self.currentProject):
                    if isinstance(o, self.lType) and isinstance(o, clr.GetClrType(ISettableName)):
                        name = IName.Name.__get__(o)
                        if not name or not self.ignoreNamedCtl.IsChecked:
                            l = layers.Lookup(o.Layer)
                            if l:
                                newName = self.nameMaskCtl.Text
                                newName = newName.Replace("{L}", l.Name)
                                newName = newName.Replace("{N}", name)
                                o.SetName = newName
                                if self.labelLinesCtl.IsChecked:
                                    self.AddLineLabel(o, newName)

                # you MUST commit the change to DB or they will all be un-done (inside the "with" scope)
                failGuard.Commit()
            OptionsManager.SetValue("AssignNameFromLayer:ignoreNamedCtl", self.ignoreNamedCtl.IsChecked)
            OptionsManager.SetValue("AssignNameFromLayer:nameMaskCtl", self.nameMaskCtl.Text)
            OptionsManager.SetValue("AssignNameFromLayer:labelLinesCtl", self.labelLinesCtl.IsChecked)            

        finally:
            self.currentProject.TransactionManager.AddEndMark(CommandGranularity.Command)
            UIEvents.RaiseAfterDataProcessing(self, UIEventArgs())
            wv.PauseGraphicsCache(False)

        cmd.CloseUICommand()

    def AddLineLabel(self, line, name):
        polySeg = line.ComputePolySeg()
        seg = clr.Reference[Segment]()
        t = clr.Reference[float]()
        pcl = clr.Reference[Point3D]()
        v = clr.Reference[Vector3D]()
        midStation = (polySeg.FirstNode.Station + polySeg.LastNode.Station) / 2.0
        if polySeg.FindPointFromStation(midStation, seg, t, pcl, v):
            container = line.GetSite()
            textObj = container.Add(clr.GetClrType(MText))
            textObj.Point0 = Point3D(pcl.X, pcl.Y)
            textStyle = self.labelStyleCtl.SelectedSnapIn
            textObj.TextString = name
            v.Rotate90(Side.Left)
            textObj.Rotation = v.Angle
            textObj.Layer = self.layerCtl.SelectedSerialNumber
            textObj.TextStyleSerial = textStyle.SerialNumber
            textObj.WidthFactor = textStyle.WidthFactor
            textObj.AddWhiteOut = textStyle.WhiteOut
            textObj.AutoFlip = textStyle.AutoFlip
            textObj.ObliqueAngle = textStyle.ObliqueAngle
            textObj.Height = self.ComputeTextHeight(textStyle, container)
            textObj.AttachPoint = CADWrapperHelper.ConvertTextJustificationToAttachmentPoint(textStyle.Justification)

    def ComputeTextHeight(self, textStyle, viewContainer):
        height = textStyle.Height
        if isinstance(viewContainer, clr.GetClrType(IViewSite)):
            plotScale = viewContainer.PlotScale
        else:
            csd,view = ISnapInHelpers.GetViewSite(viewContainer)
            plotScale = view.PlotScale
        
        if plotScale < 1.1:
            # sheet view
            if textStyle.HeightMode == TextHeightMode.Ground:
                height = height / self.worldPlotscale
            else:
                height /= 1000.0
        else:
            if textStyle.HeightMode == TextHeightMode.Paper:
                height = height * plotScale / 1000
        return height